package weather.persistence.text;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Map;
import java.util.SortedSet;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.TreeSet;

import weather.model.PunctualForecast;
import weather.persistence.ForecastException;
import weather.utils.*;

public class MyForecastReader implements ForecastReader {

    public Map<String, SortedSet<PunctualForecast>> readAll(Reader reader)
	    throws IOException, ForecastException {

	Map<String, SortedSet<PunctualForecast>> map = new TreeMap<String, SortedSet<PunctualForecast>>();
	BufferedReader buf = new BufferedReader(reader);
	PunctualForecast pf;

	while ((pf = readForecast(buf)) != null) {
	    SortedSet<PunctualForecast> set = map.get(pf.getPlace());
	    if (set != null) { // yes: then, add the punctual forecast to the city's forecasts
		set.add(pf);
	    } else // no, it's the first entry for that city: add the whole new entry
	    {
		set = new TreeSet<PunctualForecast>();
		set.add(pf);
		map.put(pf.getPlace(), set);
	    }
	}

	buf.close();
	return map;

    }

    private PunctualForecast readForecast(BufferedReader buff)
	    throws IOException, ForecastException {
	PunctualForecast pf = null;
	String line = buff.readLine();
	if (line == null || line.trim().isEmpty())
	    return null;
	// else, everything's going right..
	StringTokenizer tok = new StringTokenizer(line, ";");
	String place = tok.nextToken().trim();
	String[] data = tok.nextToken().trim().split("/");
	String[] ora = tok.nextToken().trim().split("\\.");
	String temp = tok.nextToken().trim();
	temp = temp.replace("", "");
	String um = tok.nextToken().trim();
	um = um.replace("%", "");
	String weather = tok.nextToken().trim();

	try {
	    int day = Integer.parseInt(data[0]);
	    if (day < 1 || day > 31)
		throw new ForecastException("File bad format");
	    int month = Integer.parseInt(data[1]);
	    if (month < 1 || month > 12)
		throw new ForecastException("File bad format");
	    int year = Integer.parseInt(data[2]);
	    int hour = Integer.parseInt(ora[0]);
	    if (hour < 0 || hour > 23)
		throw new ForecastException("File bad format");
	    int min = Integer.parseInt(ora[1]);
	    if (min < 0 || min > 59)
		throw new ForecastException("File bad format");
	    GregorianCalendar calendar = new GregorianCalendar();
	    calendar.clear();
	    calendar.set(year, Month.fromConventionalMonth(month)
		    .toGregorianCalendarMonth(), day, hour, min);
	    Date date = calendar.getTime();
	    int umidi = Integer.parseInt(um);
	    float temper = Float.parseFloat(temp);
	    pf = new PunctualForecast(place, date, temper, umidi, weather);
	} catch (NumberFormatException e) {
	    throw new ForecastException("File bad format" + e);
	} catch (NullPointerException e) {
	    throw new ForecastException("File bad format" + e);
	}

	return pf;
    }
}
