package weather.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;
import java.util.SortedSet;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import weather.model.Forecast;

public class WeatherFrame extends JFrame implements ActionListener,	ChangeListener {

	private static final long serialVersionUID = 1L;
	private Controller controller;
	private JComboBox cityComboBox;
	private JRadioButton puntuale, globale;
	private ButtonGroup gruppo;
	private DateSpinner date;
	private WeatherPanel wPanel;

	public WeatherFrame(Controller cont) {
		controller = cont;
		initGUI();
		bindData();
	}

	private void bindData() {
		for (String city : controller.getCities()) {
			cityComboBox.addItem(city);
		}
	}

	private void initGUI() {
		setTitle("Previsioni Meteo");
		getContentPane().setBackground(Color.WHITE);

		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setLayout(new BorderLayout(5, 5));
		setSize(900,640);

		// Controlli selezione
		JPanel leftPanel = new JPanel();
		{
			leftPanel.setLayout(new BorderLayout());
			leftPanel.setBackground(Color.WHITE);
			JPanel upperLeftPanel = new JPanel();
			{
				upperLeftPanel.setLayout(new BoxLayout(upperLeftPanel,BoxLayout.PAGE_AXIS));
				upperLeftPanel.setBackground(Color.WHITE);
				JLabel label = new JLabel("Citt:");
				label.setAlignmentX(CENTER_ALIGNMENT);
				upperLeftPanel.add(label);
				cityComboBox = new JComboBox();
				cityComboBox.addActionListener(this);
				upperLeftPanel.add(cityComboBox);

				label = new JLabel("Data:");
				label.setAlignmentX(CENTER_ALIGNMENT);
				upperLeftPanel.add(label);
				date = new DateSpinner();
				date.addChangeListener(this);
				upperLeftPanel.add(date);

				label = new JLabel("Tipo di previsione");
				label.setAlignmentX(CENTER_ALIGNMENT);
				upperLeftPanel.add(label);
				puntuale = new JRadioButton("Puntuale",true); // default pre-selection
				globale = new JRadioButton("Globale");
				gruppo = new ButtonGroup();
				gruppo.add(puntuale);
				gruppo.add(globale);
				upperLeftPanel.add(puntuale);
				upperLeftPanel.add(globale);
				puntuale.addActionListener(this);
				globale.addActionListener(this);
			}
			leftPanel.add(upperLeftPanel, BorderLayout.PAGE_START);
		}
		getContentPane().add(leftPanel, BorderLayout.LINE_START);

		JPanel rightPanel = new JPanel();
		{
			rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.PAGE_AXIS));
			JLabel previsioni = new JLabel("Previsioni richieste:");
			rightPanel.add(previsioni, BorderLayout.CENTER);
			wPanel = new WeatherPanel();
			JScrollPane scrollPane = new JScrollPane(wPanel);
			rightPanel.add(scrollPane, BorderLayout.LINE_START);
		}
		getContentPane().add(rightPanel, BorderLayout.CENTER);
	}

	private void refreshForecast() {
		// thanks to the pre-selection of the above radiobutton, either "puntuale"
		// or "globale" is always selected: however, punctual forecast is the default
		// if no buttons are selected
		String city = (String) cityComboBox.getSelectedItem();
		Date d = date.getDateValue();
		SortedSet<Forecast> forecast = controller.searchForecast(city, d, globale.isSelected());
		wPanel.setForecast(forecast, controller);
		validate();
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		refreshForecast();
	}

	@Override
	public void stateChanged(ChangeEvent arg0) {
		refreshForecast();
	}

}
